-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

local srcnode = nil;
local srcnodename = "";

local cycleindex = 0;

local labels = {};

local sFont = "sheettext";
local sFontColor = "";
local widgetText = nil;

function onInit()
	-- GET PARAMETERS
	if font then
		sFont = font[1];
	end
	if color then
		sFontColor = color[1];
	end

	local index = 0;
	table.insert(labels, index, RecursionManager.getAllRecursionText());
	index = index + 1;
	local aRecursions = RecursionManager.getRecursions();
	table.sort(aRecursions);
	for _,v in pairs(aRecursions) do
		table.insert(labels, index, v);
		index = index + 1;
	end
	
	initialize();

	-- SET ACCESS RIGHTS
	local bLocked = false;
	
	-- SET UP DATA CONNECTION
	if not sourceless then
		srcnodename = getName();
		if source and source[1] and source[1].name then
			srcnodename = source[1].name[1];
		end
	end

	if srcnodename ~= "" then
		-- DETERMINE DB READ-ONLY STATE
		local node = window.getDatabaseNode();
		if node.isReadOnly() then
			bLocked = true;
		end

		-- LINK TO DATABASE NODE, AND FUTURE UPDATES
		srcnode = node.getChild(srcnodename);
		if not srcnode then
			srcnode = node.createChild(srcnodename, "string");
			srcnode.setValue(getStringValue());
		end
		srcnode.onUpdate = update;

		-- SYNCHRONIZE DATA VALUES
		synchData();
	end

	-- Set the correct read only value
	if bLocked then
		setReadOnly(bLocked);
	end
	
	-- SET UP TEXT WIDGET
	widgetText = addTextWidget(sFont, "");
	widgetText.setPosition("center", 0, 0);

	if sFontColor and sFontColor ~= "" then
		widgetText.setColor(sFontColor);
	end

	-- UPDATE DISPLAY
	updateDisplay();

	-- REGISTER CHANGE HANDLER
	RecursionManager.addRecursionsChangedHandler(onRecursionsChanged);
end

function onRecursionsChanged(aRecursions)
	local index = 0;
	labels = { [index] = RecursionManager.getAllRecursionText() };
	index = index + 1;
	table.sort(aRecursions);
	for _,v in pairs(aRecursions) do
		if v ~= "" then
			table.insert(labels, index, v);
			index = index + 1;
		end
	end

	update();
end

function registerUpdate(nodeSource, nodeChild)
	if nodeChild.getName() == srcnodename then
		nodeSource.onChildAdded = function () end;
		nodeChild.onUpdate = update;
		srcnode = nodeChild;
		update();
	end
end

function initialize(sInitialValue)
	if sInitialValue then
		matchData(sInitialValue);
	end
end

function matchData(sValue)
	local nMatch = 0;
	for k,v in pairs(labels) do
		if v == sValue then
			nMatch = k;
		end
	end

	cycleindex = nMatch;
end

function synchData()
	local srcval = RecursionManager.getAllRecursionText();
	if srcnode then
		srcval = srcnode.getValue();
	end
	
	matchData(srcval);
end

function updateDisplay()
	widgetText.setText(labels[cycleindex]);
	if maxtextwidth and maxtextwidth[1] then
		widgetText.setMaxWidth(maxtextwidth[1]);
	end
	self.setTooltipText(labels[cycleindex]);

	if alignleft then
		local w,_ = widgetText.getSize();
		widgetText.setPosition("left", math.floor(w/2), 0);
	elseif alignright then
		local w,_ = widgetText.getSize();
		widgetText.setPosition("right", -math.floor(w/2), 0);
	end
end

function update()
	synchData();
	updateDisplay();

	if self.onValueChanged then
		self.onValueChanged();
	end
end

function getDatabaseNode()
	return srcnode;
end

function setStringValue(srcval)
	if srcnode then
		srcnode.setValue(srcval);
	else
		matchData(srcval);
		updateDisplay();
		if self.onValueChanged then
			self.onValueChanged();
		end
	end
end

function getValue()
	return labels[cycleindex];
end

function getStringValue()
	return labels[cycleindex];
end

function setDisplayColor(sColor)
	widgetText.setColor(sColor);
end

function setDisplayFont(sFont)
	widgetText.setFont(sFont);
end

function cycleLabel(bBackward)
	if bBackward then
		if cycleindex > 0 then
			cycleindex = cycleindex - 1;
		else
			cycleindex = #labels;
		end
	else
		if cycleindex < #labels then
			cycleindex = cycleindex + 1;
		else
			cycleindex = 0;
		end
	end

	if srcnode then
		srcnode.setValue(getStringValue());
	else
		updateDisplay();
		if self.onValueChanged then
			self.onValueChanged();
		end
	end
end

function onClickDown(button, x, y)
	return true;
end

function onClickRelease(button, x, y)
	if not isReadOnly() then
		cycleLabel(Input.isControlPressed());
	end
	return true;
end
